﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Diagnostics;
using System.IO;
using nMail;

namespace csmail
{
    public partial class FormMain : Form
    {
        public class MailParameter
        {
            public string Smtp;
            public string Pop3;
            public string Address;
            public bool Apop;
            public int SmtpPort;
            public int SmtpAuth;
            public int SmtpSsl;
            public int Pop3Port;
            public int Pop3Ssl;
        }
        private const string XmlFileName = "csmailparam.xml";
        private string Id = "";
        private string Password = "";
        private bool FirstFlag = true;
        private MailParameter MailParam;

        private void GetMailList()
        {
            if (Id == "")
            {
                if (!InputId())
                {
                    return;
                }
            }
            listMail.Items.Clear();
            using (nMail.Pop3 pop = new nMail.Pop3())
            {
                try
                {
                    nMail.Options.EnableConnectTimeout();
                    pop.SSL = MailParam.Pop3Ssl;
                    pop.Connect(MailParam.Pop3, MailParam.Pop3Port);
                    pop.Authenticate(Id, Password);
                    for (int no = 1; no <= pop.Count; no++)
                    {
                        pop.GetStatus(no);
                        string[] item = { pop.Subject, pop.From, pop.DateString };
                        listMail.Items.Add(new ListViewItem(item));
                    }
                    pop.Close();
                }
                catch (nMailException nex)
                {
                    if (nex.ErrorCode == nMail.Pop3.ErrorAuthenticate)
                    {
                        MessageBox.Show(string.Format("認証エラーです。\r\n{0}", nex.Message));
                        Id = "";
                        Password = "";
                    }
                    else
                    {
                        MessageBox.Show("サーバー接続時にエラーが発生しました。\r\n{0}", nex.Message);
                    }
                }
                catch (Exception ex)
                {
                    Debug.WriteLine(ex.Message);
                }
            }
        }

        public FormMain()
        {
            InitializeComponent();
        }

        private bool InputId()
        {
            FormId form = new FormId();

            form.Id = Id;
            form.Password = Password;
            if (form.ShowDialog() == DialogResult.OK)
            {
                Id = form.Id;
                Password = form.Password;
                if (Id != "" && Password != "")
                {
                    return true;
                }
            }
            return false;
        }

        private void FormMain_Load(object sender, EventArgs e)
        {
            MailParam = new MailParameter();
            MailParam.Pop3 = "";
            MailParam.Pop3Port = nMail.Pop3.StandardPortNo;
            MailParam.Pop3Ssl = 0;
            MailParam.Smtp = "";
            MailParam.SmtpPort = nMail.Smtp.StandardPortNo;
            MailParam.SmtpSsl = 0;
            MailParam.SmtpAuth = 0;
            MailParam.Address = "";
            MailParam.Apop = false;
            if (File.Exists(XmlFileName))
            {
                System.Xml.Serialization.XmlSerializer serializer = new System.Xml.Serialization.XmlSerializer(typeof(MailParameter));
                FileStream fs = new FileStream(XmlFileName, FileMode.Open);
                MailParam = (MailParameter)serializer.Deserialize(fs);
                fs.Close();
            }
            // Winsock 初期化
            nMail.Winsock.Initialize();
            // HTML パートをファイルに保存
            nMail.Options.EnableSaveHtmlFile();
        }

        private void FormMain_FormClosed(object sender, FormClosedEventArgs e)
        {
            // Winsock 終了
            nMail.Winsock.Done();
        }

        private void listMail_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (listMail.SelectedItems.Count > 0)
            {
                int no = listMail.SelectedItems[0].Index + 1;
                if (no > 0)
                {
                    try
                    {
                        nMail.Options.EnableDecodeBody();
                        nMail.Options.EnableDecodeHeader();

                        nMail.Pop3 pop = new nMail.Pop3();
                        pop.SSL = MailParam.Pop3Ssl;
                        pop.Connect(MailParam.Pop3, MailParam.Pop3Port);
                        pop.Authenticate(Id, Password);
                        pop.GetMail(no);
                        textBody.Text = pop.Body;
                        pop.Close();
                    }
                    catch (nMailException nex)
                    {
                        if (nex.ErrorCode == nMail.Pop3.ErrorAuthenticate)
                        {
                            MessageBox.Show(string.Format("認証エラーです。\r\n{0}", nex.Message));
                            Id = "";
                            Password = "";
                        }
                        else
                        {
                            MessageBox.Show("サーバー接続時にエラーが発生しました。\r\n{0}", nex.Message);
                        }
                    }
                    catch (Exception ex)
                    {
                        Debug.WriteLine(ex.Message);
                    }
                }
            }
        }

        private void FormMain_Activated(object sender, EventArgs e)
        {
            if (FirstFlag)
            {
                FirstFlag = false;
                if (MailParam.Pop3 == "" || MailParam.Pop3Port == 0)
                {
                    menuOption_Click(sender, e);
                }
                if (MailParam.Pop3 != "")
                {
                    GetMailList();
                }
            }
        }

        private void menuRedisp_Click(object sender, EventArgs e)
        {
            FirstFlag = false;
            GetMailList();
        }

        private void menuCreate_Click(object sender, EventArgs e)
        {
            FormWrite form = new FormWrite();

            form.HostName = MailParam.Smtp;
            form.From = MailParam.Address;
            form.SSL = MailParam.SmtpSsl;
            form.PortNo = MailParam.SmtpPort;
            form.Auth = MailParam.SmtpAuth;
            form.Id = Id;
            form.Password = Password;
            form.ShowDialog();
        }

        private void menuSave_Click(object sender, EventArgs e)
        {
            if (listMail.SelectedItems.Count > 0)
            {
                int no = listMail.SelectedItems[0].Index + 1;
                if (saveFileDialog.ShowDialog() == DialogResult.OK)
                {
                    if (saveFileDialog.FileName != "")
                    {
                        if (no > 0)
                        {
                            try
                            {
                                nMail.Options.DisableDecodeBodyAll();
                                nMail.Options.DisableDecodeHeader();

                                nMail.Pop3 pop = new nMail.Pop3();
                                pop.SSL = MailParam.Pop3Ssl;
                                pop.Connect(MailParam.Pop3, MailParam.Pop3Port);
                                pop.Authenticate(Id, Password);
                                pop.GetMail(no);
                                StreamWriter writer = new StreamWriter(saveFileDialog.FileName);
                                writer.Write(pop.Header);
                                writer.Write("\r\n");
                                writer.Write(pop.Body);
                                writer.Close();
                                pop.Close();
                            }
                            catch (nMailException nex)
                            {
                                if (nex.ErrorCode == nMail.Pop3.ErrorAuthenticate)
                                {
                                    MessageBox.Show(string.Format("認証エラーです。\r\n{0}", nex.Message));
                                    Id = "";
                                    Password = "";
                                }
                                else
                                {
                                    MessageBox.Show("サーバー接続時にエラーが発生しました。\r\n{0}", nex.Message);
                                }
                            }
                            catch (Exception ex)
                            {
                                Debug.WriteLine(ex.Message);
                            }
                        }
                    }
                }
            }
        }

        private void menuUnite_Click(object sender, EventArgs e)
        {
            FormUnite form = new FormUnite();

            form.ShowDialog();
        }

        private void menuDelete_Click(object sender, EventArgs e)
        {
            if (listMail.SelectedItems.Count > 0)
            {
                int no = listMail.SelectedItems[0].Index + 1;
                if (no > 0)
                {
                    try
                    {
                        nMail.Pop3 pop = new nMail.Pop3();
                        pop.SSL = MailParam.Pop3Ssl;
                        pop.Connect(MailParam.Pop3, MailParam.Pop3Port);
                        pop.Authenticate(Id, Password);
                        pop.Delete(no);
                        MessageBox.Show("メールを削除しました。");
                    }
                    catch (nMailException nex)
                    {
                        if (nex.ErrorCode == nMail.Pop3.ErrorAuthenticate)
                        {
                            MessageBox.Show(string.Format("認証エラーです。\r\n{0}", nex.Message));
                            Id = "";
                            Password = "";
                        }
                        else
                        {
                            MessageBox.Show("サーバー接続時にエラーが発生しました。\r\n{0}", nex.Message);
                        }
                    }
                    catch (Exception ex)
                    {
                        Debug.WriteLine(ex.Message);
                    }
                }
                GetMailList();
            }
        }

        private void menuExit_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void menuOption_Click(object sender, EventArgs e)
        {
            FormOption form = new FormOption();

            form.Pop3 = MailParam.Pop3;
            form.Pop3Port = MailParam.Pop3Port;
            form.Pop3Ssl = MailParam.Pop3Ssl;
            form.Smtp = MailParam.Smtp;
            form.SmtpPort = MailParam.SmtpPort;
            form.SmtpSsl = MailParam.SmtpSsl;
            form.SmtpAuth = MailParam.SmtpAuth;
            form.Address = MailParam.Address;
            form.Apop = MailParam.Apop;
            if (form.ShowDialog() == DialogResult.OK)
            {
                MailParam.Pop3 = form.Pop3;
                MailParam.Pop3Port = form.Pop3Port;
                MailParam.Pop3Ssl = form.Pop3Ssl;
                MailParam.Smtp = form.Smtp;
                MailParam.SmtpPort = form.SmtpPort;
                MailParam.SmtpSsl = form.SmtpSsl;
                MailParam.SmtpAuth = form.SmtpAuth;
                MailParam.Address = form.Address;
                MailParam.Apop = form.Apop;

                System.Xml.Serialization.XmlSerializer serializer = new System.Xml.Serialization.XmlSerializer(typeof(MailParameter));
                FileStream fs = new FileStream(XmlFileName, FileMode.Create);
                serializer.Serialize(fs, MailParam);
                fs.Close();
            }
        }

        private void menuFile_Popup(object sender, EventArgs e)
        {
            if (listMail.SelectedItems.Count == 0)
            {
                menuSave.Enabled = false;
                menuDelete.Enabled = false;
            }
            else
            {
                menuSave.Enabled = true;
                menuDelete.Enabled = true;
            }
        }

    }
}
