﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Diagnostics;
using nMail;

namespace csmail
{
    public partial class FormUnite : Form
    {
        private string Id;
        private int PartNo;

        public FormUnite()
        {
            InitializeComponent();
        }

        private void buttonFolder_Click(object sender, EventArgs e)
        {
            if (folderBrowserDialog.ShowDialog() == DialogResult.OK)
            {
                textFolder.Text = folderBrowserDialog.SelectedPath;
            }
        }

        private void buttonSave_Click(object sender, EventArgs e)
        {
            if (textFolder.Text == "")
            {
                return;
            }
            if (listFile.Items.Count == 0)
            {
                return;
            }
            using (nMail.Attachment attach = new nMail.Attachment())
            {
                attach.Path = textFolder.Text;
                try
                {
                    nMail.Options.EnableDecodeBody();
                    for (int no = 0; no < listFile.Items.Count; no++)
                    {
                        StreamReader reader = new System.IO.StreamReader((string)listFile.Items[no]);
                        string header = reader.ReadToEnd();
                        reader.Close();
                        attach.Add(header);
                    }
                    attach.Save();
                    if (attach.HtmlFile != "")
                    {
                        // 正常終了
                        if (MessageBox.Show(textFolder.Text + " に添付ファイル " + attach.FileName + " を展開しました。\r\n\r\n件名:" + attach.Subject + "\r\n本文:\r\n" + attach.Body + "\r\nHTML ファイル " + attach.HtmlFile + " を開きますか？\r\n(ウィルス等がある場合があります。ご注意下さい)", "問い合わせ", MessageBoxButtons.YesNo) == DialogResult.Yes)
                        {
                            Process.Start(attach.Path + "\\" + attach.HtmlFile);
                        }
                    }
                    else
                    {
                        // 正常終了
                        MessageBox.Show(textFolder.Text + " に添付ファイル " + attach.FileName + " を展開しました。\r\n\r\n件名:" + attach.Subject + "\r\n本文:\r\n" + attach.Body);
                    }
                    Close();
                }
                catch (nMailException nex)
                {
                    if (nex.ErrorCode == nMail.Attachment.ErrorFileOpen)
                    {
                        MessageBox.Show("添付ファイルがオープンできません。");
                    }
                    else if (nex.ErrorCode == nMail.Attachment.ErrorInvalidNo)
                    {
                        MessageBox.Show("分割されたメールの順番が正しくないか、該当しないファイルが入っています。");
                    }
                    else if (nex.ErrorCode == nMail.Attachment.ErrorPartial)
                    {
                        MessageBox.Show("分割されたメールが全て揃っていません");
                    }
                }
                catch (Exception ex)
                {
                    Debug.WriteLine(ex.Message);
                }
            }
        }

        private void buttonAdd_Click(object sender, EventArgs e)
        {
            if (openFileDialog.ShowDialog() == DialogResult.OK)
            {
                StreamReader reader = new System.IO.StreamReader(openFileDialog.FileName);
                string header = reader.ReadToEnd();
                reader.Close();

                using (nMail.Attachment attach = new nMail.Attachment())
                {
                    int no = attach.GetId(header);
                    if ((no == 0 && listFile.Items.Count == 0) || (PartNo == no))
                    {
                        if (no == 1)
                        {
                            Id = attach.PartId;
                        }
                        if (no == 0 || Id == attach.PartId)
                        {
                            listFile.Items.Add(openFileDialog.FileName);
                            PartNo++;
                            buttonSave.Enabled = true;
                        }
                        else
                        {
                            MessageBox.Show("分割されたメールデータが一致しません", "csmail", MessageBoxButtons.OK, MessageBoxIcon.Error);
                        }
                    }
                    else if (listFile.Items.Count == 0 && attach.GetHeaderField("Content-Type:", header).IndexOf("text/html") >= 0)
                    {
                        listFile.Items.Add(openFileDialog.FileName);
                        buttonSave.Enabled = true;
                    }
                    else
                    {
                        if (no < 0)
                        {
                            MessageBox.Show("分割されたメールが存在しません", "csmail", MessageBoxButtons.OK, MessageBoxIcon.Error);
                        }
                        else
                        {
                            MessageBox.Show("分割されたメールデータの順番が正しくありません", "csmail", MessageBoxButtons.OK, MessageBoxIcon.Error);
                        }
                    }
                }
            }
        }

        private void FormUnite_Load(object sender, EventArgs e)
        {
            textFolder.Text = Directory.GetCurrentDirectory();
        }
    }
}
