﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using System.IO;
using System.Diagnostics;
using nMail;

namespace csoauth
{
    public partial class FormMain : Form
    {
        private const int serviceGmail = 0;
        private const int serviceOffice365 = 1;
        private readonly string[] AuthUri = {
           "https://accounts.google.com/o/oauth2/auth",
           "https://login.microsoftonline.com/{0}/oauth2/v2.0/authorize"
        };
        private readonly string[] AuthScope = {
            "https://mail.google.com/",
            "offline_access https://outlook.office.com/SMTP.Send https://outlook.office.com/POP.AccessAsUser.All https://outlook.office.com/IMAP.AccessAsUser.All",
        };
        private readonly string[] AuthParam = {
            "access_type=offline",
            ""
        };
        private readonly string[] TokenUri = {
            "https://accounts.google.com/o/oauth2/token",
            "https://login.microsoftonline.com/{0}/oauth2/v2.0/token"
        };
        private readonly string[] Pop3Host = {
            "pop.gmail.com",
            "Outlook.office365.com"
        };
        private readonly string[] Imap4Host = {
            "imap.gmail.com",
            "Outlook.office365.com"
        };
        private readonly string[] SmtpHost = {
            "smtp.gmail.com",
            "Smtp.office365.com"
        };
        public class OAuthParameter
        {
            public int ServiceType;
            public string ClientId;
            public string ClientSecret;
            public string TenantId;
            public string RedirectUri;
            public string RefreshToken;
            public string AccessToken;
            public string MailAddress;
        }
        private const string XmlFileName = "csoauthparam.xml"; // 設定保存ファイル
        private OAuthParameter OAuthParam;	// メールサーバ等の設定
        private nMail.OAuth2 Oauth;			// nMail.DLL OAuth2 クラス
        private bool StartFlag = false;
        private string Title = "csoauth";
        private string FromReceived = "";
        private string SubjectReceived = "";

        public FormMain()
        {
            InitializeComponent();
        }

        private void FormMain_Load(object sender, EventArgs e)
        {
            OAuthParam = new OAuthParameter();
            if(File.Exists(XmlFileName))
            {
                System.Xml.Serialization.XmlSerializer serializer = new System.Xml.Serialization.XmlSerializer(typeof(OAuthParameter));
                FileStream fs = new FileStream(XmlFileName, FileMode.Open);
                OAuthParam = (OAuthParameter)serializer.Deserialize(fs);
                fs.Close();
            } 
            comboService.SelectedIndex = OAuthParam.ServiceType;
            textClientId.Text = OAuthParam.ClientId;
            textClientSecret.Text = OAuthParam.ClientSecret;
            textTenantId.Text = OAuthParam.TenantId;
            textRedirectUri.Text = OAuthParam.RedirectUri;
            textRefreshToken.Text = OAuthParam.RefreshToken;
            textAccessToken.Text = OAuthParam.AccessToken;
            textMailAddress.Text = OAuthParam.MailAddress;
            CheckEnableControl();

            // Winsock 初期化
            Winsock.Initialize();

            // 認可コード取得イベント設定
            Oauth = new nMail.OAuth2();
            Oauth.AuthorizeEvent += new System.EventHandler(AuthorizeEvent);
        }

        private delegate void AuthorizeEventDelegate(object sender, System.EventArgs e);
        // 認可コード取得イベント
        private void AuthorizeEvent(object sender, System.EventArgs e)
        {
            if(InvokeRequired)
            {
                BeginInvoke(new AuthorizeEventDelegate(AuthorizeEvent), new object[] { sender, e });
            }
            else
            {
                try
                {
                    Oauth.GetRefreshToken();
                    textRefreshToken.Text = Oauth.RefreshToken;
                    textAccessToken.Text = Oauth.AccessToken;
                    Clipboard.SetText(textRefreshToken.Text);
                }
                catch(FormatException)
                {
                    MessageBox.Show("パラメータを入力してください。", Title);
                }
                catch(InvalidOperationException)
                {
                    MessageBox.Show("リフレッシュトークンの取得に失敗しました。\r\nトークン取得URIが正しいか確認してください。", Title);
                }
                catch(nMail.nMailException nex)
                {
                    if(nex.ErrorCode == OAuth2.ErrorNoToken)
                    {
                        MessageBox.Show("リフレッシュトークンの取得に失敗しました。\r\nアカウントからログアウト後、再度実行してください。", Title);
                    }
                    else
                    {
                        MessageBox.Show(string.Format("エラーが発生しました。\r\n{0}", nex.Message), Title);
                    }
                }
                catch(Exception ex)
                {
                    Debug.WriteLine(ex.Message);
                }
                Oauth.EndAuthorize();
                StartFlag = false;
                buttonGetRefresh.Text = "リフレッシュトークン取得";
            }
        }

        private void WriteXml()
        {
            OAuthParam.ServiceType = comboService.SelectedIndex;
            OAuthParam.ClientId = textClientId.Text;
            OAuthParam.ClientSecret = textClientSecret.Text;
            OAuthParam.TenantId = textTenantId.Text;
            OAuthParam.RedirectUri = textRedirectUri.Text;
            OAuthParam.RefreshToken = textRefreshToken.Text;
            OAuthParam.AccessToken = textAccessToken.Text;
            OAuthParam.MailAddress = textMailAddress.Text;
            System.Xml.Serialization.XmlSerializer serializer = new System.Xml.Serialization.XmlSerializer(typeof(OAuthParameter));
            FileStream fs = new FileStream(XmlFileName, FileMode.Create);
            serializer.Serialize(fs, OAuthParam);
            fs.Close();
        }

        private void CheckEnableControl()
        {
            if(comboService.SelectedIndex == 1) {
                textTenantId.Enabled = true;
            } else {
                textTenantId.Enabled = false;
            }
            labelAuthUri.Text = String.Format(AuthUri[comboService.SelectedIndex], textTenantId.Text);
            labelTokenUri.Text = String.Format(TokenUri[comboService.SelectedIndex], textTenantId.Text);
        }

        private void FormMain_FormClosed(object sender, FormClosedEventArgs e)
        {
            Winsock.Done();
            WriteXml();
        }

        private void buttonGetRefresh_Click(object sender, EventArgs e)
        {
            if(StartFlag)
            {
                Oauth.EndAuthorize();
                StartFlag = false;
                buttonGetRefresh.Text = "リフレッシュトークン取得";
            }
            else
            {
                Oauth.AuthUri = labelAuthUri.Text;
                Oauth.TokenUri = labelTokenUri.Text;
                Oauth.ClientId = textClientId.Text;
                Oauth.ClientSecret = textClientSecret.Text;
                Oauth.RedirectUri = textRedirectUri.Text;
                Oauth.ResponseOk = "<html><head><meta charset=\"utf-8\"><title>取得成功</title></head><body>認可コードの取得成功</body></html>";
                Oauth.ResponseError = "<html><head><meta charset=\"utf-8\"><title>取得エラー</title></head><body>認可コードの取得エラー</body></html>";
                Oauth.Scope = AuthScope[comboService.SelectedIndex];
                Oauth.AuthParameter = AuthParam[comboService.SelectedIndex];
                Oauth.StateEnable = true;
                textRefreshToken.Text = "";
                textAccessToken.Text = "";
                try
                {
                    Oauth.StartAuthorize();
                    StartFlag = true;
                    buttonGetRefresh.Text = "中止";
                }
                catch(FormatException)
                {
                    MessageBox.Show("パラメータを入力してください。", Title);
                }
                catch(InvalidOperationException)
                {
                    MessageBox.Show("サーバーが起動できません。\nリダイレクトURIのポート番号を確認してください。", Title);
                }
                catch(Exception ex)
                {
                    Debug.WriteLine(ex.Message);
                }
            }
        }

        private void buttonGetAccess_Click(object sender, EventArgs e)
        {
            try
            {
                Oauth.TokenUri = labelTokenUri.Text;
                Oauth.ClientId = textClientId.Text;
                Oauth.ClientSecret = textClientSecret.Text;
                Oauth.RefreshToken = textRefreshToken.Text;
                Oauth.GetAccessToken();
                textAccessToken.Text = Oauth.AccessToken;
                Clipboard.SetText(textAccessToken.Text);
            }
            catch(FormatException)
            {
                MessageBox.Show("パラメータを入力してください。", Title);
            }
            catch(InvalidOperationException)
            {
                MessageBox.Show("アクセストークンの取得に失敗しました。", Title);
            }
            catch(nMail.nMailException nex)
            {
                MessageBox.Show(string.Format("エラーが発生しました。\r\n{0}", nex.Message), Title);
            }
            catch(Exception ex)
            {
                Debug.WriteLine(ex.Message);
            }
        }

        private void comboService_SelectedIndexChanged(object sender, EventArgs e)
        {
            CheckEnableControl();
        }

        private void textTenantId_TextChanged(object sender, EventArgs e)
        {
            CheckEnableControl();
        }

        private void buttonReceive_Click(object sender, EventArgs e)
        {
            if(textMailAddress.Text == "")
            {
                MessageBox.Show("メールアドレスを入力してください。", Title);
                return;
            }
            if(textAccessToken.Text == "")
            {
                MessageBox.Show("アクセストークンを取得してください。", Title);
                return;
            }
            if(MessageBox.Show("メールを受信しますか？", Title, MessageBoxButtons.YesNo) == DialogResult.Yes) {
                using (nMail.Pop3 pop = new nMail.Pop3())
                {
                    try
                    {
                        nMail.Options.EnableConnectTimeout();
                        pop.SSL = nMail.Pop3.TLS1;
                        pop.Port = nMail.Pop3.StandardSslPortNo;
                        pop.Connect(Pop3Host[comboService.SelectedIndex]);
                        pop.Authenticate(textMailAddress.Text, textAccessToken.Text, nMail.Pop3.AuthOAuth2);
                        if((int)numericMailNo.Value <= pop.Count) {
                            pop.GetMail((int)numericMailNo.Value);
                            textTo.Text = pop.From;
                            textSubject.Text = pop.Subject;
                            textBody.Text = pop.Body;
                            FromReceived = pop.From;
                            SubjectReceived = pop.Subject;
                        } else {
                            MessageBox.Show("指定の番号のメールが存在しません。", Title);
                        }
                        pop.Close();
                    }
                    catch (nMailException nex)
                    {
                        if (nex.ErrorCode == nMail.Pop3.ErrorAuthenticate)
                        {
                            MessageBox.Show(string.Format("認証エラーです。\r\n{0}", nex.Message), Title);
                        }
                        else
                        {
                            MessageBox.Show(string.Format("サーバー接続時にエラーが発生しました。\r\n{0}", nex.Message), Title);
                        }
                    }
                    catch (Exception ex)
                    {
                        Debug.WriteLine(ex.Message);
                    }
                }
            }
        }

        private void buttonSend_Click(object sender, EventArgs e)
        {
            if(textMailAddress.Text == "")
            {
                MessageBox.Show("メールアドレスを入力してください。", Title);
                return;
            }
            if(textAccessToken.Text == "")
            {
                MessageBox.Show("アクセストークンを取得してください。", Title);
                return;
            }
            if (textTo.Text == "")
            {
                MessageBox.Show("宛先を入力してください", Title);
                return;
            }
            if (textSubject.Text == "")
            {
                MessageBox.Show("件名を入力してください", Title);
                return;
            }
            if (textBody.Text == "")
            {
                MessageBox.Show("本文を入力してください", Title);
                return;
            }
            if (textTo.Text == FromReceived && textSubject.Text == SubjectReceived)
            {
                MessageBox.Show("受信したメールをそのまま返送してしまいます。\n宛先や件名を変更してください。", Title);
                return;
            }
            if(MessageBox.Show("メールを送信しますか？", Title, MessageBoxButtons.YesNo) == DialogResult.Yes) {
                using (nMail.Smtp smtp = new nMail.Smtp())
                {
                    try
                    {
                        smtp.SSL = nMail.Smtp.TLS1;
                        smtp.Port = nMail.Smtp.StandardSslPortNo;
                        if(comboService.SelectedIndex == serviceOffice365)
                        {
                            smtp.SSL |= nMail.Smtp.STARTTLS;
                            smtp.Port = nMail.Smtp.SubmissionPortNo;
                        }
                        smtp.Connect(SmtpHost[comboService.SelectedIndex]);
                        smtp.Authenticate(textMailAddress.Text, textAccessToken.Text, nMail.Smtp.AuthOAuth2);
                        smtp.SendMail(textTo.Text, textMailAddress.Text, textSubject.Text, textBody.Text);
                        smtp.Close();
                        MessageBox.Show("送信成功しました。", Title);
                    }
                    catch (nMailException nex)
                    {
                        MessageBox.Show(String.Format("{0:d}", nex.ErrorCode), Title);
                    }
                    catch (Exception ex)
                    {
                        Debug.WriteLine(ex.Message);
                    }
                }
            }
        }

        private void buttonReceiveImap4_Click(object sender, EventArgs e)
        {
            if(textMailAddress.Text == "")
            {
                MessageBox.Show("メールアドレスを入力してください。", Title);
                return;
            }
            if(textAccessToken.Text == "")
            {
                MessageBox.Show("アクセストークンを取得してください。", Title);
                return;
            }
            if(MessageBox.Show("メールを受信しますか？", Title, MessageBoxButtons.YesNo) == DialogResult.Yes) {
                using (nMail.Imap4 imap = new nMail.Imap4())
                {
                    try
                    {
                        nMail.Options.EnableConnectTimeout();
                        imap.SSL = nMail.Imap4.TLS1;
                        imap.Port = nMail.Imap4.StandardSslPortNo;
                        imap.Connect(Imap4Host[comboService.SelectedIndex]);
                        imap.Authenticate(textMailAddress.Text, textAccessToken.Text, nMail.Imap4.AuthOAuth2);
                        imap.SelectMailBox(textMailBox.Text);
                        if((int)numericMailNo.Value <= imap.Count) {
                            imap.GetMail((uint)numericMailNo.Value);
                            textTo.Text = imap.From;
                            textSubject.Text = imap.Subject;
                            textBody.Text = imap.Body;
                            FromReceived = imap.From;
                            SubjectReceived = imap.Subject;
                        } else {
                            MessageBox.Show("指定の番号のメールが存在しません。", Title);
                        }
                        imap.Close();
                    }
                    catch (nMailException nex)
                    {
                        if (nex.ErrorCode == nMail.Imap4.ErrorAuthenticate)
                        {
                            MessageBox.Show(string.Format("認証エラーです。\r\n{0}", nex.Message), Title);
                        }
                        else
                        {
                            MessageBox.Show(string.Format("サーバー接続時にエラーが発生しました。\r\n{0}", nex.Message), Title);
                        }
                    }
                    catch (Exception ex)
                    {
                        Debug.WriteLine(ex.Message);
                    }
                }
            }
        }
    }
}
